<?php
/**
 * BIG FISH Payment Gateway payment method class
 *
 * @package paymentMethod
 * @copyright (c) 2015, BIG FISH Internet-technology Ltd. (http://bigfish.hu)
 */

namespace BigFish\Helpers;


/**
 * BIG FISH Payment Gateway ProviderData
 *
 */
class ProviderData {
	private $saferpayPaymentMethods = array(
		'AMEX' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_AMEX,
		'DIRECTDEBIT' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_DIRECTDEBIT,
		'INVOICE' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_INVOICE,
		'BONUS' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_BONUS,
		'DINERS' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_DINERS,
		'EPRZELEWY' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_EPRZELEWY,
		'EPS' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_EPS,
		'GIROPAY' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_GIROPAY,
		'IDEAL' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_IDEAL,
		'JCB' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_JCB,
		'MAESTRO' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_MAESTRO,
		'MASTERCARD' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_MASTERCARD,
		'MYONE' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_MYONE,
		'PAYPAL' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_PAYPAL,
		'POSTCARD' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_POSTCARD,
		'POSTFINANCE' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_POSTFINANCE,
		'SAFERPAYTEST' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_SAFERPAYTEST,
		'SOFORT' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_SOFORT,
		'VISA' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_VISA,
		'VPAY' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_VPAY
	);

	private $saferpayWallets = array(
		'MASTERPASS' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY_WALLET_MASTERPASS
	);

	private $wirecardPaymentTypes = array(
		'SELECT' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_SELECT,
		'BANCONTACT_MISTERCASH' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_BANCONTACT_MISTERCASH,
		'CCARD' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_CCARD,
		'CCARD-MOTO' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_CCARD_MOTO,
		'EKONTO' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_EKONTO,
		'EPAY_BG' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_EPAY_BG,
		'EPS' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_EPS,
		'GIROPAY' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_GIROPAY,
		'IDL' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_IDL,
		'MONETA' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_MONETA,
		'MPASS' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_MPASS,
		'PRZELEWY24' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_PRZELEWY24,
		'PAYPAL' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_PAYPAL,
		'PBX' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_PBX,
		'POLI' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_POLI,
		'PSC' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_PSC,
		'QUICK' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_QUICK,
		'SEPA-DD' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_SEPADD,
		'SKRILLDIRECT' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_SKRILLDIRECT,
		'SKRILLWALLET' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_SKRILLWALLET,
		'SOFORTUEBERWEISUNG' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_SOFORTUEBERWEISUNG,
		'TATRAPAY' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_TATRAPAY,
		'TRUSTLY' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_TRUSTLY,
		'TRUSTPAY' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_TRUSTPAY,
		'VOUCHER' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY_VOUCHER,
	);

	private $_provider = array();
	private $_KhbCardPocketId = array();
	private $_MkbSzepCafeteriaId = array();
	private $_OtpCardPocketId = array();
	private $_requestAutoCommit = array();
	private $_CreditCard = false;
	private $_escalionOneClickPayment = false;
	private $_otpSimpleOneClickPayment = false;
	private $_saferpayOneClickPayment = false;

	private $_saferpayPaymentMethods = array();
	private $_saferpayWallets = array();
	private $_qpayPaymentTypes = '';

	public function __construct()
	{
		$this->setAllowedProviders();
	}

	/**
	 * Set provider
	 *
	 */
	public function setAllowedProviders()
	{
		$this->_provider = array();
		$providers = explode(', ', MODULE_PAYMENT_BFPG_PROVIDERS);

		foreach((array)$providers as $provider) {
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_CIB) {
				$this->_provider['CIB'] = $this->getProvidersData('CIB');
			}
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_KHB_CREDITCARD) {
				$this->_provider['KHB'] = $this->getProvidersData('KHB');
			}
			if (
				$provider == MODULE_PAYMENT_BFPG_PROVIDER_KHB_SZEP_SZALLAS ||
				$provider == MODULE_PAYMENT_BFPG_PROVIDER_KHB_SZEP_VENDEGLATAS ||
				$provider == MODULE_PAYMENT_BFPG_PROVIDER_KHB_SZEP_REKREACIO
			) {
				$this->_provider['KHBSZEP'] = $this->getProvidersData('KHBSZEP');

				if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_KHB_SZEP_SZALLAS) {
					$this->_KhbCardPocketId['KHBSZEP1'] = $this->getPocketsData('KHBSZEP1');
				}
				if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_KHB_SZEP_VENDEGLATAS) {
					$this->_KhbCardPocketId['KHBSZEP2'] = $this->getPocketsData('KHBSZEP2');
				}
				if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_KHB_SZEP_REKREACIO) {
					$this->_KhbCardPocketId['KHBSZEP3'] = $this->getPocketsData('KHBSZEP3');
				}
			}
			if (
				$provider == MODULE_PAYMENT_BFPG_PROVIDER_OTP_CREDITCARD ||
				$provider == MODULE_PAYMENT_BFPG_PROVIDER_OTP_SZEP_SZALLAS ||
				$provider == MODULE_PAYMENT_BFPG_PROVIDER_OTP_SZEP_VENDEGLATAS ||
				$provider == MODULE_PAYMENT_BFPG_PROVIDER_OTP_SZEP_REKREACIO
			) {
				$this->_provider['OTP'] = $this->getProvidersData('OTP');

				if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_OTP_CREDITCARD) {
					$this->_OtpCardPocketId['OTP'] = $this->getPocketsData('OTP');
				}
				if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_OTP_SZEP_SZALLAS) {
					$this->_OtpCardPocketId['OTP09'] = $this->getPocketsData('OTP09');
				}
				if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_OTP_SZEP_VENDEGLATAS) {
					$this->_OtpCardPocketId['OTP07'] = $this->getPocketsData('OTP07');
				}
				if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_OTP_SZEP_REKREACIO) {
					$this->_OtpCardPocketId['OTP08'] = $this->getPocketsData('OTP08');
				}
			}
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_OTPAY) {
				$this->_provider['OTPay'] = $this->getProvidersData('OTPay');
			}
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_OTP2) {
				$this->_provider['OTP2'] = $this->getProvidersData('OTP2');
				$this->_CreditCard = true;
			}
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_ESCALION) {
				$this->_provider['Escalion'] = $this->getProvidersData('Escalion');
			}
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_QPAY) {
				$this->_provider['QPAY'] = $this->getProvidersData('QPAY');
				$this->_qpayPaymentTypes = $this->getQpayPaymentTypes();
			}
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_UNICREDIT) {
				$this->_provider['Unicredit'] = $this->getProvidersData('Unicredit');
			}
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_FHB) {
				$this->_provider['FHB'] = $this->getProvidersData('FHB');
			}
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_PAYPAL) {
				$this->_provider['PayPal'] = $this->getProvidersData('PayPal');
			}
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_PAYU2) {
				$this->_provider['PayU2'] = $this->getProvidersData('PayU2');
			}
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_SMS) {
				$this->_provider['SMS'] = $this->getProvidersData('SMS');
			}
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_SOFORT) {
				$this->_provider['Sofort'] = $this->getProvidersData('Sofort');
			}
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_BORGUN) {
				$this->_provider['Borgun'] = $this->getProvidersData('Borgun');
			}
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_OTPMASTERPASS) {
				$this->_provider['OTPayMP'] = $this->getProvidersData('OTPayMP');
			}
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_OTPMULTIPONT) {
				$this->_provider['OTPMultipont'] = $this->getProvidersData('OTPMultipont');
			}
			if (
				$provider == MODULE_PAYMENT_BFPG_PROVIDER_MKB_SZEP_SZALLAS ||
				$provider == MODULE_PAYMENT_BFPG_PROVIDER_MKB_SZEP_VENDEGLATAS ||
				$provider == MODULE_PAYMENT_BFPG_PROVIDER_MKB_SZEP_REKREACIO
			) {
				$this->_provider['MKBSZEP'] = $this->getProvidersData('MKBSZEP');

				if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_MKB_SZEP_SZALLAS) {
					$this->_MkbSzepCafeteriaId['MKBSZEP1111'] = $this->getPocketsData('MKBSZEP1111');
				}
				if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_MKB_SZEP_VENDEGLATAS) {
					$this->_MkbSzepCafeteriaId['MKBSZEP2222'] = $this->getPocketsData('MKBSZEP2222');
				}
				if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_MKB_SZEP_REKREACIO) {
					$this->_MkbSzepCafeteriaId['MKBSZEP3333'] = $this->getPocketsData('MKBSZEP3333');
				}
			}
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_OTPSIMPLE) {
				$this->_provider['OTPSimple'] = $this->getProvidersData('OTPSimple');
			}
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_PAYSAFECARD) {
				$this->_provider['PSC'] = $this->getProvidersData('PSC');
			}
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_OTPSIMPLEWIRE) {
				$this->_provider['OTPSimpleWire'] = $this->getProvidersData('OTPSimpleWire');
			}
			if ($provider == MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY) {
				$this->_provider['Saferpay'] = $this->getProvidersData('Saferpay');
				$this->_saferpayPaymentMethods = $this->getSaferpayPaymentMethods();
				$this->_saferpayWallets = $this->getSaferpayWallets();
			}
		}
		
		if (MODULE_PAYMENT_BFPG_AUTOCOMMIT_OTP == MODULE_PAYMENT_BFPG_TEXT_DISABLE) {
			$this->_requestAutoCommit['OTP'] = 0;
		}
		if (MODULE_PAYMENT_BFPG_AUTOCOMMIT_OTP2 == MODULE_PAYMENT_BFPG_TEXT_DISABLE) {
			$this->_requestAutoCommit['OTP2'] = 0;
		}
		if (MODULE_PAYMENT_BFPG_AUTOCOMMIT_OTPAY == MODULE_PAYMENT_BFPG_TEXT_DISABLE) {
			$this->_requestAutoCommit['OTPay'] = 0;
		}
		if (MODULE_PAYMENT_BFPG_AUTOCOMMIT_QPAY == MODULE_PAYMENT_BFPG_TEXT_DISABLE) {
			$this->_requestAutoCommit['QPAY'] = 0;
		}
		if (MODULE_PAYMENT_BFPG_AUTOCOMMIT_UNICREDIT == MODULE_PAYMENT_BFPG_TEXT_DISABLE) {
			$this->_requestAutoCommit['Unicredit'] = 0;
		}
		if (MODULE_PAYMENT_BFPG_AUTOCOMMIT_FHB == MODULE_PAYMENT_BFPG_TEXT_DISABLE) {
			$this->_requestAutoCommit['FHB'] = 0;
		}
		if (MODULE_PAYMENT_BFPG_AUTOCOMMIT_PAYU2 == MODULE_PAYMENT_BFPG_TEXT_DISABLE) {
			$this->_requestAutoCommit['PayU2'] = 0;
		}

		$this->_escalionOneClickPayment = (MODULE_PAYMENT_BFPG_ONECLICKPAYMENT_ESCALION == MODULE_PAYMENT_BFPG_TEXT_ENABLE);
		$this->_otpSimpleOneClickPayment = (MODULE_PAYMENT_BFPG_ONECLICKPAYMENT_OTPSIMPLE == MODULE_PAYMENT_BFPG_TEXT_ENABLE);
		$this->_saferpayOneClickPayment = (MODULE_PAYMENT_BFPG_ONECLICKPAYMENT_SAFERPAY == MODULE_PAYMENT_BFPG_TEXT_ENABLE);
	}

	/**
	 * Get var
	 *
	 * @param $var
	 *
	 * @return mixed
	 */
	public function getVar($var)
	{
		return $this->{'_' . $var};
	}

	/**
	 * Get providers data
	 *
	 * @param string $provider
	 *
	 * @return array
	 */
	public function getProvidersData($provider = '')
	{
		$data = array();

		$data['CIB'] = array(
			'value' => 'CIB',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_CIB,
		);
		$data['KHB'] = array(
			'value' => 'KHB',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_KHB_CREDITCARD,
		);
		$data['KHBSZEP'] = array(
			'value' => 'KHBSZEP',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_KHB_SZEP,
		);
		$data['OTP'] = array(
			'value' => 'OTP',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_OTP,
		);
		$data['OTPay'] = array(
			'value' => 'OTPay',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_OTPAY,
		);
		$data['OTP2'] = array(
			'value' => 'OTP2',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_OTP2,
		);
		$data['Escalion'] = array(
			'value' => 'Escalion',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_ESCALION,
		);
		$data['QPAY'] = array(
			'value' => 'QPAY',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_QPAY,
		);
		$data['Unicredit'] = array(
			'value' => 'Unicredit',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_UNICREDIT,
		);
		$data['FHB'] = array(
			'value' => 'FHB',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_FHB,
		);
		$data['PayPal'] = array(
			'value' => 'PayPal',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_PAYPAL,
		);
		$data['PayU2'] = array(
			'value' => 'PayU2',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_PAYU2,
		);
		$data['SMS'] = array(
			'value' => 'SMS',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_SMS,
		);
		$data['Sofort'] = array(
			'value' => 'Sofort',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_SOFORT,
		);
		$data['Borgun'] = array(
			'value' => 'Borgun',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_BORGUN,
		);
		$data['OTPMultipont'] = array(
			'value' => 'OTPMultipont',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_OTPMULTIPONT,
		);
		$data['OTPayMP'] = array(
			'value' => 'OTPayMP',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_OTPMASTERPASS,
		);
		$data['MKBSZEP'] = array(
			'value' => 'MKBSZEP',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_MKB_SZEP,
		);
		$data['PSC'] = array(
			'value' => 'PSC',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_PAYSAFECARD,
		);
		$data['OTPSimple'] = array(
			'value' => 'OTPSimple',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_OTPSIMPLE,
		);
		$data['OTPSimpleWire'] = array(
			'value' => 'OTPSimpleWire',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_OTPSIMPLEWIRE,
		);
		$data['Saferpay'] = array(
			'value' => 'Saferpay',
			'title' => MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY,
		);

		if ($provider == '') {
			return $data;
		} else {
			return $data[$provider];
		}
	}

	public function getPocketsData($pocket = '')
	{
		$data = array();

		$data['KHBSZEP1'] = array(
			'value' => '1',
			'title' => MODULE_PAYMENT_BFPG_TEXT_ACCOMODATION,
			'szep' => true,
			'name' => MODULE_PAYMENT_BFPG_PROVIDER_KHB_SZEP_SZALLAS,
		);
		$data['KHBSZEP2'] = array(
			'value' => '2',
			'title' => MODULE_PAYMENT_BFPG_TEXT_HOSPITALITY,
			'szep' => true,
			'name' => MODULE_PAYMENT_BFPG_PROVIDER_KHB_SZEP_VENDEGLATAS,
		);
		$data['KHBSZEP3'] = array(
			'value' => '3',
			'title' => MODULE_PAYMENT_BFPG_TEXT_RECREATION,
			'szep' => true,
			'name' => MODULE_PAYMENT_BFPG_PROVIDER_KHB_SZEP_REKREACIO,
		);
		$data['OTP'] = array(
			'value' => '',
			'title' => MODULE_PAYMENT_BFPG_TEXT_CREDITCARD,
			'name' => MODULE_PAYMENT_BFPG_PROVIDER_OTP_CREDITCARD,
		);
		$data['OTP09'] = array(
			'value' => '09',
			'title' => MODULE_PAYMENT_BFPG_TEXT_ACCOMODATION,
			'szep' => true,
			'name' => MODULE_PAYMENT_BFPG_PROVIDER_OTP_SZEP_SZALLAS,
		);
		$data['OTP07'] = array(
			'value' => '07',
			'title' => MODULE_PAYMENT_BFPG_TEXT_HOSPITALITY,
			'szep' => true,
			'name' => MODULE_PAYMENT_BFPG_PROVIDER_OTP_SZEP_VENDEGLATAS,
		);
		$data['OTP08'] = array(
			'value' => '08',
			'title' => MODULE_PAYMENT_BFPG_TEXT_RECREATION,
			'szep' => true,
			'name' => MODULE_PAYMENT_BFPG_PROVIDER_OTP_SZEP_REKREACIO,
		);
		$data['MKBSZEP1111'] = array(
			'value' => '1111',
			'title' => MODULE_PAYMENT_BFPG_TEXT_ACCOMODATION,
			'szep' => true,
			'name' => MODULE_PAYMENT_BFPG_PROVIDER_MKB_SZEP_SZALLAS,
		);
		$data['MKBSZEP2222'] = array(
			'value' => '2222',
			'title' => MODULE_PAYMENT_BFPG_TEXT_HOSPITALITY,
			'szep' => true,
			'name' => MODULE_PAYMENT_BFPG_PROVIDER_MKB_SZEP_VENDEGLATAS,
		);
		$data['MKBSZEP3333'] = array(
			'value' => '3333',
			'title' => MODULE_PAYMENT_BFPG_TEXT_RECREATION,
			'szep' => true,
			'name' => MODULE_PAYMENT_BFPG_PROVIDER_MKB_SZEP_REKREACIO,
		);

		if ($pocket == '') {
			return $data;
		} else {
			return $data[$pocket];
		}
	}

	public function getSaferpayPaymentMethods() {
		$data = array();
		$saferpayPaymentMethods = explode(', ', MODULE_PAYMENT_BFPG_SAFERPAY_PAYMENT_METHODS);

		foreach ($saferpayPaymentMethods as $paymentMethod) {
			$data[$paymentMethod] = array(
				'value' => $paymentMethod,
				'title' => $paymentMethod
			);
		}

		return $data;
	}

	public function getSaferpayWallets() {
		$data = array();

		$saferpayWallets = explode(', ', MODULE_PAYMENT_BFPG_SAFERPAY_WALLETS);
		foreach ($saferpayWallets as $wallet) {
			$data[$wallet] = array(
				'value' => $wallet,
				'title' => $wallet
			);
		}

		return $data;
	}

	public function getQpayPaymentTypes() {
		$data = '';

		$wirecardPaymentTypes = explode(', ', MODULE_PAYMENT_BFPG_QPAY_PAYMENT_TYPES);
		foreach ($wirecardPaymentTypes as $paymentType) {
			$search = array_search($paymentType, $this->wirecardPaymentTypes);
			if ($search !== null) {
				$data = $search;
				break;
			}
		}

		return $data;
	}

	public function getSaferpayPaymentMethodsArray() {
		return $this->saferpayPaymentMethods;
	}

	public function getSaferpayWalletsArray() {
		return $this->saferpayWallets;
	}

	public function getWirecardPaymentTypesArray() {
		return $this->wirecardPaymentTypes;
	}
} 