<?php
/**
 * BIG FISH Payment Gateway payment method class
 *
 * @package paymentMethod
 * @copyright (c) 2015, BIG FISH Internet-technology Ltd. (http://bigfish.hu)
 * @version: 1.1.0
 */
require 'includes/application_top.php';

// define our general functions used application-wide

include_once(DIR_WS_CLASSES . 'order.php');
include_once(DIR_FS_CATALOG . 'BigFish/Autoload.php');
include_once(DIR_FS_CATALOG . 'BigFish/define.php');
include_once(DIR_WS_LANGUAGES . $_SESSION['language'] . '/modules/payment/bigfish_paymentgateway.php');
include_once(DIR_WS_LANGUAGES . $_SESSION['language'] . '/checkout_process.php');
include_once(DIR_WS_FUNCTIONS . 'html_output.php');

BigFish\Autoload::register();

use BigFish\Helpers\ProviderData;
use BigFish\PaymentGateway;
use BigFish\PaymentGateway\Exception as PaymentGateway_Exception;
use BigFish\PaymentGateway\Request\Result as ResultRequest;
use BigFish\PaymentGateway\Config;

global $db;

/**
 * Go to checkout success page
 *
 * @param $transactionData
 */
function goToCheckoutSuccess($transactionData) {
	$_SESSION['cart']->reset(true);
	unset($_SESSION[BIGFISH_CODENAME]);
	require(DIR_WS_CLASSES . 'order_total.php');
	$order_total_modules = new order_total;
	$order_total_modules->clear_posts();
	if(!isset($_GET['notification'])) {
		addSessionToPaymentMessage($transactionData);
		zen_redirect(zen_href_link(FILENAME_CHECKOUT_SUCCESS, '', 'SSL'));
	}

}
/**
 * add payment message to session
 *
 * @param $transactionData
 */
function addSessionToPaymentMessage($transactionData) {
	$_SESSION['payment_method_messages'] = '';
	if(!empty($transactionData['Anum'])) {
		$_SESSION['payment_method_messages'] .= sprintf(
			MODULE_PAYMENT_BFPG_CHECKOUT_SUCCESS_ANUM,
			$transactionData['Anum']
		);
	}
	if(!empty($transactionData['ProviderTransactionId'])) {
		$_SESSION['payment_method_messages'] .= sprintf(
			MODULE_PAYMENT_BFPG_CHECKOUT_SUCCESS_PTID,
			$transactionData['ProviderTransactionId']
		);
	}
}

/**
 * update order status
 *
 * @param $orderId
 * @param $status
 * @param string $comment
 * @param int $notification
 * @param array $transactionData
 */
function updateOrderStatus($orderId, $status, $comment = '', $notification = 1, $transactionData = array()) {
	global $db, $order;
	$sql = "UPDATE " . TABLE_ORDERS .
		" SET orders_status = " . (int)$status .
		" WHERE orders_id = " . (int)$orderId;
	$db->Execute($sql, false, false, 0, true);

	$sql_data_array = array(
		'orders_id' => (int)$orderId,
		'orders_status_id' => (int)$status,
		'date_added' => 'now()',
		'comments' => $comment,
		'customer_notified' => (int)$notification,
	);
	zen_db_perform(TABLE_ORDERS_STATUS_HISTORY, $sql_data_array);

	$order = new order($orderId);
	$order->notify('NOTIFY_ORDER_CART_ORDERSTATUS', $sql_data_array);

	if($notification) {
		$orders_status_array = array();
		$sql = "SELECT orders_status_id, orders_status_name" .
			" FROM " . TABLE_ORDERS_STATUS .
			" WHERE language_id = '" . (int)$_SESSION['languages_id'] . "' ORDER BY orders_status_id";
		$orders_status = $db->Execute($sql, false, false, 0, true);
		while (!$orders_status->EOF) {
			$orders_statuses[] = array('id' => $orders_status->fields['orders_status_id'],
				'text' => $orders_status->fields['orders_status_name'] . ' [' . $orders_status->fields['orders_status_id'] . ']');
			$orders_status_array[$orders_status->fields['orders_status_id']] = $orders_status->fields['orders_status_name'];
			$orders_status->MoveNext();
		}
		$sql = "SELECT customers_name, customers_email_address, orders_status, date_purchased" .
			" FROM " . TABLE_ORDERS .
			" WHERE orders_id = '" . (int)$orderId . "'";
		$check_status = $db->Execute($sql, false, false, 0, true);
		//send emails
		$message = EMAIL_TEXT_ORDER_NUMBER . ' ' . $orderId . "\n\n";
		if(!empty($transactionData['Anum'])) {
			$message .= EMAIL_TEXT_ANUM . $transactionData['Anum'] . "\n\n";
		}
		if(!empty($transactionData['ProviderTransactionId'])) {
			$message .= EMAIL_TEXT_PTID . $transactionData['ProviderTransactionId'] . "\n\n";
		}
		$message .= EMAIL_TEXT_INVOICE_URL . ' ' . zen_href_link(FILENAME_CATALOG_ACCOUNT_HISTORY_INFO, 'order_id=' . $orderId, 'SSL') . "\n\n" .
			EMAIL_TEXT_DATE_ORDERED . ' ' . zen_date_long($check_status->fields['date_purchased']) . "\n\n" .
			EMAIL_TEXT_STATUS_UPDATED . sprintf(EMAIL_TEXT_STATUS_LABEL, $orders_status_array[$status] ) .
			EMAIL_TEXT_STATUS_PLEASE_REPLY;

		$html_msg['EMAIL_CUSTOMERS_NAME']    = $check_status->fields['customers_name'];
		$html_msg['EMAIL_TEXT_ORDER_NUMBER'] = EMAIL_TEXT_ORDER_NUMBER . ' ' . $orderId;
		$html_msg['EMAIL_TEXT_ANUM'] = EMAIL_TEXT_ANUM . ' ' . $transactionData['Anum'];
		$html_msg['EMAIL_TEXT_PTID'] = EMAIL_TEXT_PTID . ' ' . $transactionData['ProviderTransactionId'];
		$html_msg['EMAIL_TEXT_INVOICE_URL']  = '<a href="' . zen_href_link(FILENAME_CATALOG_ACCOUNT_HISTORY_INFO, 'order_id=' . $orderId, 'SSL') .'">'.str_replace(':','',EMAIL_TEXT_INVOICE_URL).'</a>';
		$html_msg['EMAIL_TEXT_DATE_ORDERED'] = EMAIL_TEXT_DATE_ORDERED . ' ' . zen_date_long($check_status->fields['date_purchased']);
		$html_msg['EMAIL_TEXT_STATUS_UPDATED'] = str_replace('\n','', EMAIL_TEXT_STATUS_UPDATED);
		$html_msg['EMAIL_TEXT_STATUS_LABEL'] = str_replace('\n','', sprintf(EMAIL_TEXT_STATUS_LABEL, $orders_status_array[$status] ));
		$html_msg['EMAIL_TEXT_NEW_STATUS'] = $orders_status_array[$status];
		$html_msg['EMAIL_TEXT_STATUS_PLEASE_REPLY'] = str_replace('\n','', EMAIL_TEXT_STATUS_PLEASE_REPLY);

		zen_mail(
			$check_status->fields['customers_name'],
			$check_status->fields['customers_email_address'],
			EMAIL_TEXT_SUBJECT . ' #' . $orderId,
			$message,
			STORE_NAME,
			EMAIL_FROM,
			$html_msg,
			'order_status'
		);
	}
}

if(isset($_GET['notification'])) {
	$json = file_get_contents('php://input');
	$notificationData = json_decode($json, true);
	$_GET['TransactionId'] = $notificationData['commonData']['transactionId'];
}
$sql = "SELECT *
	FROM " . BIGFISH_CODENAME . "_session
	WHERE session_id = :sessionID";
$sql = $db->bindVars($sql, ':sessionID', $_GET[zen_session_name()], 'string');
$stored_session = $db->Execute($sql);
if($stored_session->fields['saved_session']) {
	$_SESSION = unserialize(base64_decode($stored_session->fields['saved_session']));
}
$sql = "SELECT *
	  FROM " . BIGFISH_CODENAME . "
	  WHERE TransactionId=:TransactionId ";
$sql = $db->bindVars($sql, ':TransactionId', $_GET['TransactionId'], 'string');
/** @var object $result */
$result = $db->Execute($sql, 1);
if(!$result->fields) {
	error_log(sprintf(MODULE_PAYMENT_BFPG_TEXT_ERROR_NO_DATA, $_GET['TransactionId']));
	header('HTTP/1.1 404 Not Found');
	exit;
}
$oldTransactionData = $result->fields;

$configData = array(
	'apiKey' => MODULE_PAYMENT_BFPG_APIKEY,
	'testMode' => (MODULE_PAYMENT_BFPG_TESTMODE == MODULE_PAYMENT_BFPG_TEXT_TEST ? true : false),
	'storeName' => MODULE_PAYMENT_BFPG_STORENAME,
);
if($oldTransactionData['provider'] == 'OTP2') {
	$configData['encryptPublicKey'] = MODULE_PAYMENT_BFPG_PUBLICKEY_OTP2;
}
$config = new Config($configData);
PaymentGateway::setConfig($config);

try {
	/** @var object $response */
	$response = PaymentGateway::result(new ResultRequest($_GET['TransactionId']));
} catch(PaymentGateway_Exception $e) {
	global $messageStack;
	$messageStack->add_session('checkout_payment', $e->getMessage() . '<!-- [' . BIGFISH_CODENAME . '] -->', 'error');
	if(!isset($_GET['notification'])) {
		zen_redirect(zen_href_link(FILENAME_CHECKOUT_PAYMENT, '', 'SSL', true, false));
	} else {
		error_log($e->getMessage());
	}
}

$transactionData = array();
$transactionData['ResultCode'] = $response->ResultCode;
$transactionData['ResultMessage'] = $response->ResultMessage;
$transactionData['ProviderTransactionId'] = $response->ProviderTransactionId;
$transactionData['AutoCommit'] = ($response->AutoCommit == 'false') ? 0 : 1;
$transactionData['CommitState'] = $response->CommitState;
if(isset($response->Anum)) {
	$transactionData['Anum'] = $response->Anum;
} else {
	$transactionData['Anum'] = null;
}
$newTransactionData = array_merge($oldTransactionData, $transactionData);
$otherData = get_object_vars($response);
unset(
	$otherData['OrderId'],
	$otherData['UserId'],
	$otherData['TransactionId'],
	$otherData['ResultCode'],
	$otherData['ResultMessage'],
	$otherData['ProviderTransactionId'],
	$otherData['AutoCommit'],
	$otherData['CommitState'],
	$otherData['Anum']
);
$transactionData['OtherData'] = json_encode($otherData);

$sql = "
UPDATE
". BIGFISH_CODENAME ."
SET
Anum=:Anum,
ResultCode=:ResultCode,
ResultMessage=:ResultMessage,
ProviderTransactionId=:ProviderTransactionId,
AutoCommit=:AutoCommit,
CommitState=:CommitState,
OtherData=:OtherData
WHERE
TransactionId = :TransactionId ";
$sql = $db->bindVars($sql, ':Anum', $transactionData['Anum'], 'string');
$sql = $db->bindVars($sql, ':ResultCode', $transactionData['ResultCode'], 'string');
$sql = $db->bindVars($sql, ':ResultMessage', $transactionData['ResultMessage'], 'string');
$sql = $db->bindVars($sql, ':ProviderTransactionId', $transactionData['ProviderTransactionId'], 'string');
$sql = $db->bindVars($sql, ':AutoCommit', $transactionData['AutoCommit'], 'integer');
$sql = $db->bindVars($sql, ':CommitState', $transactionData['CommitState'], 'string');
$sql = $db->bindVars($sql, ':OtherData', $transactionData['OtherData'], 'string');
$sql = $db->bindVars($sql, ':TransactionId', $response->TransactionId, 'string');
$db->Execute($sql, false, false, 0, true);

$sql = "SELECT *
	  FROM " . BIGFISH_CODENAME . "
	  WHERE TransactionId=:TransactionId ";
$sql = $db->bindVars($sql, ':TransactionId', $response->TransactionId, 'string');
$result = $db->Execute($sql, 1, false, 0, true);
$transactionData = $result->fields;

$sql = "SELECT *
	  FROM " . TABLE_ORDERS . "
	  WHERE orders_id=:orders_id ";
$sql = $db->bindVars($sql, ':orders_id', (int)$transactionData['orders_id'], 'integer');
$orderResult = $db->Execute($sql, 1, false, 0, true);

$ProviderDataMethod = new ProviderData();
$ProviderData['provider'] = $ProviderDataMethod->getVar('provider');
$ProviderData['KhbCardPocketId'] = $ProviderDataMethod->getVar('KhbCardPocketId');
$ProviderData['MkbSzepCafeteriaId'] = $ProviderDataMethod->getVar('MkbSzepCafeteriaId');
$ProviderData['OtpCardPocketId'] = $ProviderDataMethod->getVar('OtpCardPocketId');

$comments = 'Provider: ' . $ProviderData['provider'][$transactionData['provider']]['title'];
if(!empty($transactionData['OtpCardPocketId'])) {
	$pocketName = $ProviderData['OtpCardPocketId'][$transactionData['provider'] . $transactionData['OtpCardPocketId']]['title'];
	$comments .= "\n" . MODULE_PAYMENT_BFPG_TEXT_POCKET . ': ' . $transactionData['OtpCardPocketId'] . '(' . $pocketName . ')';
}
if(!empty($transactionData['KhbCardPocketId'])) {
	$pocketName = $ProviderData['KhbCardPocketId'][$transactionData['provider'] . $transactionData['KhbCardPocketId']]['title'];
	$comments .= "\n" . MODULE_PAYMENT_BFPG_TEXT_POCKET . ': ' . $transactionData['KhbCardPocketId'] . '(' . $pocketName . ')';
}
if(!empty($transactionData['MkbSzepCafeteriaId'])) {
	$pocketName = $ProviderData['MkbSzepCafeteriaId'][$transactionData['provider'] . $transactionData['MkbSzepCafeteriaId']]['title'];
	$comments .= "\n" . MODULE_PAYMENT_BFPG_TEXT_POCKET . ': ' . $transactionData['MkbSzepCafeteriaId'] . '(' . $pocketName . ')';
}
foreach(get_object_vars($response) as $key => $value) {
	$comments .= "\n" . $key . ': ' . $value;
}

$_SESSION['payment_method_messages'] = '';
if($response->ResultCode == PaymentGateway::RESULT_CODE_SUCCESS) {
	// Update order status
	if(
		!in_array($orderResult->fields['orders_status'], array(
			MODULE_PAYMENT_BFPG_SUCCESS_STATUS_ID,
			MODULE_PAYMENT_BFPG_WAITING_STATUS_ID))
		||
		$newTransactionData['CommitState'] != $oldTransactionData['CommitState']
	) {
		if($newTransactionData['AutoCommit']==1) {
			updateOrderStatus(
				$oldTransactionData['orders_id'],
				MODULE_PAYMENT_BFPG_SUCCESS_STATUS_ID,
				$comments,
				1,
				$transactionData
			);
		} else {
			updateOrderStatus(
				$oldTransactionData['orders_id'],
				MODULE_PAYMENT_BFPG_WAITING_STATUS_ID,
				$comments,
				1,
				$transactionData
			);
		}
	}
	goToCheckoutSuccess($transactionData);
} else {
	if($newTransactionData['ResultCode'] != $oldTransactionData['ResultCode']) {
		updateOrderStatus(
			$oldTransactionData['orders_id'],
			MODULE_PAYMENT_BFPG_CANCELLED_STATUS_ID,
			$comments,
			1,
			$transactionData
		);
	}
	global $messageStack;
	$messageStack->add_session('checkout_payment', $response->ResultMessage . '<!-- [' . BIGFISH_CODENAME . '] -->', 'error');
	if(!isset($_GET['notification'])) {
		zen_redirect(zen_href_link(FILENAME_CHECKOUT_PAYMENT, '', 'SSL', true, false));
	}
}