<?php

/**
 * BIG FISH Payment Gateway payment method class
 *
 * @package paymentMethod
 * @copyright (c) 2015, BIG FISH Internet-technology Ltd. (http://bigfish.hu)
 */
include_once(DIR_FS_CATALOG . 'BigFish/Autoload.php');
include_once(DIR_FS_CATALOG . 'BigFish/define.php');
BigFish\Autoload::register();

use BigFish\Helpers\ProviderData;
use BigFish\PaymentGateway;
use BigFish\PaymentGateway\Exception as PaymentGateway_Exception;
use BigFish\PaymentGateway\Request\Init as InitRequest;
use BigFish\PaymentGateway\Request\Start as StartRequest;
use BigFish\PaymentGateway\Request\Close as CloseRequest;
use BigFish\PaymentGateway\Request\Details as DetailsRequest;
use BigFish\PaymentGateway\Config;

/**
 * BIG FISH Payment Gateway payment method class
 *
 */
class bigfish_paymentgateway
{
	var $code, $title, $description, $enabled, $provider, $processButton;

	var $providerData = array();

	/**
	 * log
	 *
	 * @param $contents
	 */
	function log($contents)
	{
		error_log($contents);
	}

	/**
	 * bigfish_paymentgateway
	 * class constructor
	 *
	 * @param string $provider
	 */
	function __construct($provider = '')
	{
		global $order;
		$this->provider = $provider;
		$this->code = BIGFISH_CODENAME;
		$this->title = MODULE_PAYMENT_BFPG_TEXT_TITLE;
		if (MODULE_PAYMENT_BFPG_STORENAME == '') {
			$this->title .= MODULE_PAYMENT_BFPG_TEXT_NOTCONFIGURED;
		} elseif (MODULE_PAYMENT_BFPG_APIKEY == '' ||
			!preg_match('/^\w{5}-\w{5}-\w{5}-\w{5}-\w{5}$/', MODULE_PAYMENT_BFPG_APIKEY)
		) {
			$this->title .= MODULE_PAYMENT_BFPG_TEXT_APIKEY_WARNING;
		}

		if (MODULE_PAYMENT_BFPG_TESTMODE == MODULE_PAYMENT_BFPG_TEXT_TEST) {
			$this->title .= MODULE_PAYMENT_BFPG_TEXT_TESTMODE;
		}
		$this->description = MODULE_PAYMENT_BFPG_TEXT_DESCRIPTION;
		$this->sort_order = MODULE_PAYMENT_BFPG_SORT_ORDER;

		if (MODULE_PAYMENT_BFPG_STATUS == MODULE_PAYMENT_BFPG_TEXT_TRUE) {
			$this->enabled = true;
		} else {
			$this->enabled = false;
		}

		if ((int)MODULE_PAYMENT_BFPG_ORDER_STATUS_ID > 0) {
			$this->order_status = MODULE_PAYMENT_BFPG_ORDER_STATUS_ID;
		}

		if (is_object($order)) {
			$this->update_status();
		}

	}

	/**
	 * update_status
	 */
	function update_status()
	{
		global $db;
		/** @var object $order */
		global $order;

		// check zone
		if ($this->enabled == true && (int)MODULE_PAYMENT_BFPG_ZONE > 0) {
			$check_flag = false;
			/** @var object $check */
			$check = $db->Execute(
				"select zone_id from " . TABLE_ZONES_TO_GEO_ZONES . " where geo_zone_id = '" .
				(int)MODULE_PAYMENT_BFPG_ZONE . "'" . " and zone_country_id = '" .
				(int)$order->billing['country']['id'] . "'" . " order by zone_id"
			);
			while(!$check->EOF) {
				if ($check->fields['zone_id'] < 1) {
					$check_flag = true;
					break;
				} elseif ($check->fields['zone_id'] == $order->billing['zone_id']) {
					$check_flag = true;
					break;
				}
				$check->MoveNext();
			}

			if ($check_flag == false) {
				$this->enabled = false;
			}
		}

		// check currency
		$currencies = array_map('trim', explode(",", MODULE_PAYMENT_BFPG_CURRENCIES));

		if (array_search($order->info['currency'], $currencies) === false) {
			$this->enabled = false;
		}

		// check that api key is not blank
		if (!MODULE_PAYMENT_BFPG_APIKEY OR !strlen(MODULE_PAYMENT_BFPG_APIKEY)) {
			$this->enabled = false;
		}

	}

	/**
	 * javascript validation
	 * form submit js check
	 *
	 * @return string
	 */
	function javascript_validation()
	{
		$js = "
if (payment_value == '" . $this->code . "') {
	var {$this->code} = new Array();
	{$this->code}['provider'] = document.getElementById('{$this->code}-provider');
	{$this->code}['providerSelected'] = {$this->code}['provider'].options[{$this->code}['provider'].selectedIndex].value;
	if ({$this->code}['providerSelected'] == 'OTP2') {
		{$this->code}['OtpCardNumber'] = document.checkout_payment['{$this->code}-OtpCardNumber'].value;
		{$this->code}['OtpExpiration'] = document.checkout_payment['{$this->code}-OtpExpiration'].value;
		{$this->code}['OtpCvc'] = document.checkout_payment['{$this->code}-OtpCvc'].value;
		if (
			{$this->code}['OtpCardNumber'] == '' ||
			{$this->code}['OtpCardNumber'].length < " . CC_NUMBER_MIN_LENGTH . "
		) {
			error_message = error_message + '" . MODULE_PAYMENT_BFPG_TEXT_ERROR_CC_NUMBER . "';
			error = 1;
		}
		if (
			{$this->code}['OtpExpiration'] == '' ||
			{$this->code}['OtpExpiration'].length != 4
		) {
			error_message = error_message + '" . MODULE_PAYMENT_BFPG_TEXT_ERROR_CC_EXP . "';
			error = 1;
		}
		if (
			{$this->code}['OtpCvc'] == '' ||
			{$this->code}['OtpCvc'].length < " . MODULE_PAYMENT_BFPG_CC_CVV_MIN_LENGTH . " ||
			{$this->code}['OtpCvc'].length > " . MODULE_PAYMENT_BFPG_CC_CVV_MAX_LENGTH . "
		) {
			error_message = error_message + '" . MODULE_PAYMENT_BFPG_TEXT_ERROR_CC_CVV . "';
			error = 1;
		}
	}
}
";

		return $js;
	}

	/**
	 * selection
	 * form element
	 *
	 * @return array
	 */
	function selection()
	{
		$this->loadProviderData();
		$default['provider'] = $_SESSION[$this->code]['provider'];
		$default['OtpCardPocketId'] = $_SESSION[$this->code]['OtpCardPocketId'];
		$default['KhbCardPocketId'] = $_SESSION[$this->code]['KhbCardPocketId'];
		$default['MkbSzepCafeteriaId'] = $_SESSION[$this->code]['MkbSzepCafeteriaId'];
		$default['OtpCardNumber'] = $_SESSION[$this->code]['OtpCardNumber'];
		$default['OtpExpiration'] = $_SESSION[$this->code]['OtpExpiration'];
		$default['OtpCvc'] = $_SESSION[$this->code]['OtpCvc'];
		$default['EscalionOneClickPayment'] = $_SESSION[$this->code]['EscalionOneClickPayment'];
		$default['OTPSimpleOneClickPayment'] = $_SESSION[$this->code]['OTPSimpleOneClickPayment'];
		$default['SaferpayOneClickPayment'] = $_SESSION[$this->code]['SaferpayOneClickPayment'];

		$selection = array(
			'id' => $this->code,
			'module' => $this->title,
			'fields' => array(
				array(
					'field' => '<script>' . str_replace(
						'###CODE###',
						$this->code,
						file_get_contents(DIR_FS_CATALOG . 'BigFish/js/selection.js')
					) . '</script>',
				),
			),
		);
		if ($this->providerData['countProvider'] > 1) {
			$providers[] = array('id' => '', 'text' => MODULE_PAYMENT_BFPG_TEXT_PLEASECHOOSE);
			foreach((array)$this->providerData['provider'] as $value) {
				$providers[] = array('id' => $value['value'], 'text' => $value['title']);
			}
			$selection['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_PROVIDERS,
				'field' => zen_draw_pull_down_menu(
					$this->code . '-provider',
					$providers,
					$default['provider'],
					'id="' . $this->code . '-provider"'
				),
				'tag' => $this->code . '-provider',
			);
		} elseif ($this->providerData['countProvider'] == 1) {
			$value = current($this->providerData['provider']);
			$selection['fields'][] = array(
				'field' => zen_draw_hidden_field(
					$this->code . '-provider',
					$value['value'],
					'id="' . $this->code . '-provider"'
				),
			);
		}

		if ($this->providerData['countOtpCardPocketId'] > 1) {
			$pockets = array();
			foreach((array)$this->providerData['OtpCardPocketId'] as $value) {
				$pockets[] = array('id' => $value['value'], 'text' => $value['title']);
			}
			$selection['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_POCKET,
				'field' => zen_draw_pull_down_menu(
					$this->code . '-OtpCardPocketId',
					$pockets,
					$default['OtpCardPocketId'],
					'id="' . $this->code . '-OtpCardPocketId" style="display:none" disabled'
				),
				'tag' => $this->code . '-OtpCardPocketId" style="display:none',
			);
		} elseif ($this->providerData['countOtpCardPocketId'] == 1) {
			$value = current($this->providerData['OtpCardPocketId']);
			if ($value['value'] != '') {
				$selection['fields'][] = array(
					'field' => zen_draw_hidden_field(
						$this->code . '-OtpCardPocketId',
						$value['value'],
						'id="' . $this->code . '-OtpCardPocketId" disabled'
					),
				);
			}
		}

		if ($this->providerData['countKhbCardPocketId'] > 1) {
			$pockets = array();
			foreach((array)$this->providerData['KhbCardPocketId'] as $value) {
				$pockets[] = array('id' => $value['value'], 'text' => $value['title']);
			}
			$selection['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_POCKET,
				'field' => zen_draw_pull_down_menu(
					$this->code . '-KhbCardPocketId',
					$pockets,
					$default['KhbCardPocketId'],
					'id="' . $this->code . '-KhbCardPocketId" style="display:none" disabled'
				),
				'tag' => $this->code . '-KhbCardPocketId" style="display:none',
			);
		} elseif ($this->providerData['countKhbCardPocketId'] == 1) {
			$value = current($this->providerData['KhbCardPocketId']);
			$selection['fields'][] = array(
				'field' => zen_draw_hidden_field(
					$this->code . '-KhbCardPocketId',
					$value['value'],
					'id="' . $this->code . '-KhbCardPocketId" disabled'
				),
			);
		}

		if ($this->providerData['countMkbSzepCafeteriaId'] > 1) {
			$pockets = array();
			foreach((array)$this->providerData['MkbSzepCafeteriaId'] as $value) {
				$pockets[] = array('id' => $value['value'], 'text' => $value['title']);
			}
			$selection['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_POCKET,
				'field' => zen_draw_pull_down_menu(
					$this->code . '-MkbSzepCafeteriaId',
					$pockets,
					$default['MkbSzepCafeteriaId'],
					'id="' . $this->code . '-MkbSzepCafeteriaId" style="display:none" disabled'
				),
				'tag' => $this->code . '-MkbSzepCafeteriaId" style="display:none',
			);
		} elseif ($this->providerData['countMkbSzepCafeteriaId'] == 1) {
			$value = current($this->providerData['MkbSzepCafeteriaId']);
			$selection['fields'][] = array(
				'field' => zen_draw_hidden_field(
					$this->code . '-MkbSzepCafeteriaId',
					$value['value'],
					'id="' . $this->code . '-MkbSzepCafeteriaId" disabled'
				),
			);
		}

		if ($this->providerData['CreditCard']) {
			$selection['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_CARDNUMBER,
				'field' => zen_draw_input_field(
					$this->code . '-OtpCardNumber',
					$default['OtpCardNumber'],
					'id="' . $this->code . '-OtpCardNumber" style="display:none" disabled'
				),
				'tag' => $this->code . '-OtpCardNumber" style="display:none',
			);
			$selection['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_EXPIRATION,
				'field' => zen_draw_input_field(
					$this->code . '-OtpExpiration',
					$default['OtpExpiration'],
					'id="' . $this->code . '-OtpExpiration" style="display:none" disabled'
				),
				'tag' => $this->code . '-OtpExpiration" style="display:none',
			);
			$selection['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_CVV,
				'field' => zen_draw_input_field(
					$this->code . '-OtpCvc',
					$default['OtpCvc'],
					'id="' . $this->code . '-OtpCvc" style="display:none" disabled'
				),
				'tag' => $this->code . '-OtpCvc" style="display:none',
			);
		}

		if ($this->providerData['EscalionOneClickPayment']) {
			$selection['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_ONECLICKPAYMENT_ESCALION,
				'field' => zen_draw_checkbox_field(
					$this->code . '-EscalionOneClickPayment',
					$default['EscalionOneClickPayment'],
					false,
					'id="' . $this->code . '-EscalionOneClickPayment" style="display:none" disabled'
				),
				'tag' => $this->code . '-EscalionOneClickPayment" style="display:none',
			);
		}
		if ($this->providerData['OTPSimpleOneClickPayment']) {
			$selection['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_ONECLICKPAYMENT_OTPSIMPLE,
				'field' => zen_draw_checkbox_field(
					$this->code . '-OTPSimpleOneClickPayment',
					$default['OTPSimpleOneClickPayment'],
					false,
					'id="' . $this->code . '-OTPSimpleOneClickPayment" style="display:none" disabled'
				),
				'tag' => $this->code . '-OTPSimpleOneClickPayment" style="display:none',
			);
		}
		if ($this->providerData['SaferpayOneClickPayment']) {
			$selection['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_ONECLICKPAYMENT_SAFERPAY,
				'field' => zen_draw_checkbox_field(
					$this->code . '-SaferpayOneClickPayment',
					$default['SaferpayOneClickPayment'],
					false,
					'id="' . $this->code . '-SaferpayOneClickPayment" style="display:none" disabled'
				),
				'tag' => $this->code . '-SaferpayOneClickPayment" style="display:none',
			);
		}
		if ($this->providerData['SaferpayPaymentMethods']) {
			foreach((array)$this->providerData['SaferpayPaymentMethods'] as $value) {
				$selection['fields'][] = array(
					'field' => zen_draw_hidden_field(
						$this->code . '-SaferpayPaymentMethods[]',
						$value['value'],
						false,
						'id="' . $this->code . '-SaferpayPaymentMethods" style="display:none" disabled'
					),
					'tag' => $this->code . '-SaferpayPaymentMethods" style="display:none',
				);
			}
		}
		if ($this->providerData['SaferpayWallets']) {
			foreach((array)$this->providerData['SaferpayWallets'] as $value) {
				$selection['fields'][] = array(
					'field' => zen_draw_hidden_field(
						$this->code . '-SaferpayWallets[]',
						$value['value'],
						false,
						'id="' . $this->code . '-SaferpayWallets" style="display:none" disabled'
					),
					'tag' => $this->code . '-SaferpayWallets" style="display:none',
				);
			}
		}
		if ($this->providerData['QPAYPaymentTypes']) {
			$selection['fields'][] = array(
				'field' => zen_draw_hidden_field(
					$this->code . '-QPAYPaymentTypes',
					(string)$this->providerData['QPAYPaymentTypes'],
					false,
					'id="' . $this->code . '-QPAYPaymentTypes" style="display:none" disabled'
				),
				'tag' => $this->code . '-QPAYPaymentTypes" style="display:none',
			);
		}

		return $selection;
	}

	/**
	 * pre confirmation check
	 * called before step 3
	 */
	function pre_confirmation_check()
	{
		global $messageStack;
		$this->loadProviderData();
		$_SESSION[$this->code]['provider'] =  $_POST[$this->code . '-provider'];
		$_SESSION[$this->code]['KhbCardPocketId'] =  $_POST[$this->code . '-KhbCardPocketId'];
		$_SESSION[$this->code]['MkbSzepCafeteriaId'] =  $_POST[$this->code . '-MkbSzepCafeteriaId'];
		$_SESSION[$this->code]['OtpCardNumber'] =  $_POST[$this->code . '-OtpCardNumber'];
		$_SESSION[$this->code]['OtpExpiration'] =  $_POST[$this->code . '-OtpExpiration'];
		$_SESSION[$this->code]['OtpCvc'] =  $_POST[$this->code . '-OtpCvc'];
		$_SESSION[$this->code]['OtpCardPocketId'] =  $_POST[$this->code . '-OtpCardPocketId'];
		$_SESSION[$this->code]['EscalionOneClickPayment'] =  $_POST[$this->code . '-EscalionOneClickPayment'];
		$_SESSION[$this->code]['OTPSimpleOneClickPayment'] =  $_POST[$this->code . '-OTPSimpleOneClickPayment'];
		$_SESSION[$this->code]['SaferpayOneClickPayment'] =  $_POST[$this->code . '-SaferpayOneClickPayment'];
		$_SESSION[$this->code]['SaferpayPaymentMethods'] =  $_POST[$this->code . '-SaferpayPaymentMethods'];
		$_SESSION[$this->code]['SaferpayWallets'] =  $_POST[$this->code . '-SaferpayWallets'];
		$_SESSION[$this->code]['QPAYPaymentTypes'] =  $_POST[$this->code . '-QPAYPaymentTypes'];

		$error = array();
		if (!isset($_POST[$this->code . '-provider']) ||
			!isset($this->providerData['provider'][$_POST[$this->code . '-provider']])
		) {
			$error[] = MODULE_PAYMENT_BFPG_TEXT_UNAUTHORIZED_PAYMENT;
		}
		if ($_POST[$this->code . '-provider'] == 'OTP2' &&
			MODULE_PAYMENT_BFPG_TESTMODE == MODULE_PAYMENT_BFPG_TEXT_PRODUCTION
		) {
			include(DIR_WS_CLASSES . 'cc_validation.php');
			$cc_validation = new cc_validation();
			$result = $cc_validation->validate(
				$_POST[$this->code . '-OtpCardNumber'],
				substr($_POST[$this->code . '-OtpExpiration'], 0, 2),
				substr($_POST[$this->code . '-OtpExpiration'], 2)
			);
			switch($result) {
				case -1:
					$error[] = sprintf(TEXT_CCVAL_ERROR_UNKNOWN_CARD, substr($cc_validation->cc_number, 0, 4));
					break;
				case -2:
				case -3:
				case -4:
					$error[] = TEXT_CCVAL_ERROR_INVALID_DATE;
					break;
				case false:
					$error[] = TEXT_CCVAL_ERROR_INVALID_NUMBER;
					break;
			}
			if (strlen($_POST[$this->code . '-OtpCvc']) < MODULE_PAYMENT_BFPG_CC_CVV_MIN_LENGTH ||
				strlen($_POST[$this->code . '-OtpCvc']) > MODULE_PAYMENT_BFPG_CC_CVV_MAX_LENGTH
			) {
				$error[] = MODULE_PAYMENT_BFPG_TEXT_ERROR_CC_CVV;
			}

		}

		if (!empty($error)) {
			$messageStack->add_session(
				'checkout_payment',
				implode('<br />', $error) . '<!-- [' . $this->code . '] -->',
				'error'
			);
			zen_redirect(zen_href_link(FILENAME_CHECKOUT_PAYMENT, '', 'SSL', true, false));
		}
	}

	/**
	 * confirmation
	 * called upon requesting step 3
	 *
	 * @return array
	 */
	function confirmation()
	{
		$this->loadProviderData();
		$postProvider = $_POST[$this->code . '-provider'];

		$confirmation = array(
			'fields' => array(
				array(
					'title' => MODULE_PAYMENT_BFPG_TEXT_PROVIDER,
					'field' => $this->providerData['provider'][$postProvider]['title'],
				),
			)
		);
		$this->processButton = zen_draw_hidden_field(
			$this->code . '-provider',
			$_POST[$this->code . '-provider']
		);
		if (isset($_POST[$this->code . '-OtpCardPocketId'])) {
			$confirmation['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_POCKET,
				'field' => $this->providerData['OtpCardPocketId'][$postProvider .
				$_POST[$this->code . '-OtpCardPocketId']]['title'],
			);
			$this->processButton .= zen_draw_hidden_field(
				$this->code . '-OtpCardPocketId',
				$_POST[$this->code . '-OtpCardPocketId']
			);
		}
		if (isset($_POST[$this->code . '-KhbCardPocketId'])) {
			$confirmation['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_POCKET,
				'field' => $this->providerData['KhbCardPocketId'][$postProvider .
				$_POST[$this->code . '-KhbCardPocketId']]['title'],
			);
			$this->processButton .= zen_draw_hidden_field(
				$this->code . '-KhbCardPocketId',
				$_POST[$this->code . '-KhbCardPocketId']
			);
		}
		if (isset($_POST[$this->code . '-MkbSzepCafeteriaId'])) {
			$confirmation['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_POCKET,
				'field' => $this->providerData['MkbSzepCafeteriaId'][$postProvider .
				$_POST[$this->code . '-MkbSzepCafeteriaId']]['title'],
			);
			$this->processButton .= zen_draw_hidden_field(
				$this->code . '-MkbSzepCafeteriaId',
				$_POST[$this->code . '-MkbSzepCafeteriaId']
			);
		}
		if ($postProvider == 'OTP2') {
			$confirmation['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_CARDNUMBER,
				'field' => $_POST[$this->code . '-OtpCardNumber'],
			);
			$this->processButton .= zen_draw_hidden_field(
				$this->code . '-OtpCardNumber',
				$_POST[$this->code . '-OtpCardNumber']
			);
			$confirmation['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_EXPIRATION,
				'field' => $_POST[$this->code . '-OtpExpiration'],
			);
			$this->processButton .= zen_draw_hidden_field(
				$this->code . '-OtpExpiration',
				$_POST[$this->code . '-OtpExpiration']
			);
			$confirmation['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_CVV,
				'field' => '***',
			);
			$this->processButton .= zen_draw_hidden_field(
				$this->code . '-OtpCvc',
				$_POST[$this->code . '-OtpCvc']
			);
		}
		if (isset($_POST[$this->code . '-EscalionOneClickPayment'])) {
			$confirmation['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_ONECLICKPAYMENT_ESCALION,
				'field' =>  $this->providerData['EscalionOneClickPayment'][$postProvider . $_POST[$this->code . '-EscalionOneClickPayment']]['title'],
			);
			$this->processButton .= zen_draw_hidden_field(
				$this->code . '-EscalionOneClickPayment',
				$_POST[$this->code . '-EscalionOneClickPayment']
			);
		}
		if (isset($_POST[$this->code . '-OTPSimpleOneClickPayment'])) {
			$confirmation['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_ONECLICKPAYMENT_OTPSIMPLE,
				'field' =>  $this->providerData['OTPSimpleOneClickPayment'][$postProvider . $_POST[$this->code . '-OTPSimpleOneClickPayment']]['title'],
			);
			$this->processButton .= zen_draw_hidden_field(
				$this->code . '-OTPSimpleOneClickPayment',
				$_POST[$this->code . '-OTPSimpleOneClickPayment']
			);
		}
		if (isset($_POST[$this->code . '-SaferpayOneClickPayment'])) {
			$confirmation['fields'][] = array(
				'title' => MODULE_PAYMENT_BFPG_TEXT_ONECLICKPAYMENT_SAFERPAY,
				'field' =>  $this->providerData['SaferpayOneClickPayment'][$postProvider . $_POST[$this->code . '-SaferpayOneClickPayment']]['title'],
			);
			$this->processButton .= zen_draw_hidden_field(
				$this->code . '-SaferpayOneClickPayment',
				$_POST[$this->code . '-SaferpayOneClickPayment']
			);
		}
		if (isset($_POST[$this->code . '-SaferpayPaymentMethods'])) {
			foreach ($_POST[$this->code . '-SaferpayPaymentMethods'] as $paymentMethod) {
				$this->processButton .= zen_draw_hidden_field(
					$this->code . '-SaferpayPaymentMethods[]',
					$paymentMethod
				);
			}
		}
		if (isset($_POST[$this->code . '-SaferpayWallets'])) {
			foreach ($_POST[$this->code . '-SaferpayWallets'] as $wallet) {
				$this->processButton .= zen_draw_hidden_field(
					$this->code . '-SaferpayWallets[]',
					$wallet
				);
			}
		}
		if (isset($_POST[$this->code . '-QPAYPaymentTypes'])) {
			$this->processButton .= zen_draw_hidden_field(
				$this->code . '-QPAYPaymentTypes',
				$_POST[$this->code . '-QPAYPaymentTypes']
			);
		}

		return $confirmation;
	}

	/**
	 * process button
	 * called upon requesting step 3 (after confirmation above)
	 *
	 * @return mixed
	 */
	function process_button()
	{
		return $this->processButton;
	}

	/**
	 * before process
	 * called upon clicking confirm
	 *
	 * @return bool
	 */
	function before_process()
	{
		return false;
	}

	/**
	 * after process
	 * called upon clicking confirm (after before_process and after the order is created)
	 *
	 * @return bool
	 */
	function after_process()
	{
		/** @var object $order */
		global $insert_id, $order, $db;

		$this->loadProviderData();
		$data = array();
		$data['provider'] = $_POST[$this->code . '-provider'];
		$data['responseUrl'] = HTTP_SERVER . '/' . BIGFISH_CALLBACK . '?' . zen_session_name() . '=' . zen_session_id();
		$data['notificationUrl'] = $data['responseUrl'] . '&notification=1';
		$total = $order->info['total'] * $order->info['currency_value'];
		if ($order->info['currency'] == 'HUF') {
			$data['amount'] = (int)$total;
		} else {
			$data['amount'] = (float)$total;
		}
		$data['currency'] = $order->info['currency'];
		$data['orderId'] = $insert_id;
		$data['userId'] = $_SESSION['customer_id'];
		$data['language'] = strtoupper($_SESSION['languages_code']);
		$data['autoCommit'] = $this->providerData['requestAutoCommit'][$data['provider']] === 0 ? false : true;
		$data['MkbSzepCafeteriaId'] = $_POST[$this->code . '-MkbSzepCafeteriaId'];
		$data['OtpCardNumber'] = $_POST[$this->code . '-OtpCardNumber'];
		$data['OtpExpiration'] = $_POST[$this->code . '-OtpExpiration'];
		$data['OtpCvc'] = $_POST[$this->code . '-OtpCvc'];
		$data['OtpCardPocketId'] = $_POST[$this->code . '-OtpCardPocketId'];
		$data['EscalionOneClickPayment'] = $_POST[$this->code . '-EscalionOneClickPayment'];
		$data['OTPSimpleOneClickPayment'] = $_POST[$this->code . '-OTPSimpleOneClickPayment'];
		$data['SaferpayOneClickPayment'] = $_POST[$this->code . '-SaferpayOneClickPayment'];

		if ($data['provider'] == PaymentGateway::PROVIDER_KHB_SZEP) {
			$data['extra']['KhbCardPocketId'] = $_POST[$this->code . '-KhbCardPocketId'];
		} else if ($data['provider'] == PaymentGateway::PROVIDER_SAFERPAY) {
			$data['extra']['SaferpayPaymentMethods'] = $_POST[$this->code . '-SaferpayPaymentMethods'];
			$data['extra']['SaferpayWallets'] = $_POST[$this->code . '-SaferpayWallets'];
		} else if ($data['provider'] == PaymentGateway::PROVIDER_WIRECARD_QPAY) {
			$data['extra']['QpayPaymentType'] = $_POST[$this->code . '-QPAYPaymentTypes'];
		}

		$this->setPaymentGatewayConfig($data['provider'], !empty($data['OtpCardPocketId']));
		try {
			$request = new InitRequest();
			$request->setProviderName($data['provider'])
				->setResponseUrl($data['responseUrl'])
				->setAmount(number_format($data['amount'], ($data['currency'] == 'HUF' ? 0 : 2), '.', ''))
				->setCurrency($data['currency'])
				->setOrderId($data['orderId'])
				->setUserId($data['userId'])
				->setLanguage($data['language'])
				->setAutoCommit($data['autoCommit'])
				->setModuleName(BIGFISH_MODULE_NAME)
				->setModuleVersion(BIGFISH_MODULE_VERSION);
			if (!empty($data['OtpCardPocketId'])) {
				$request->setOtpCardPocketId($data['OtpCardPocketId']);
			}
			if ($data['provider'] == 'MKBSZEP') {
				$request->setMkbSzepCafeteriaId($data['MkbSzepCafeteriaId'])
					->setGatewayPaymentPage(true);

			}
			if ($data['provider'] == 'OTP2') {
				$request->setOtpCardNumber($data['OtpCardNumber'])->setOtpExpiration($data['OtpExpiration'])->setOtpCvc(
					$data['OtpCvc']
				);
			}
			if (isset($data['extra']) && is_array($data['extra'])) {
				$request->setExtra($data['extra']);
			} else {
				$request->setExtra(array());
			}

			if ((isset($data['EscalionOneClickPayment']) && (int)$data['EscalionOneClickPayment']) ||
				(isset($data['OTPSimpleOneClickPayment']) && (int)$data['OTPSimpleOneClickPayment']) ||
				(isset($data['SaferpayOneClickPayment']) && (int)$data['SaferpayOneClickPayment'])) {
				$request->setOneClickPayment(true);
			}
			$request->setNotificationUrl($data['notificationUrl']);

			/** @var object $response */
			$response = PaymentGateway::init($request);
			if (!($response->ResultCode == PaymentGateway::RESULT_CODE_SUCCESS && $response->TransactionId)) {
				global $messageStack;
				$messageStack->add_session(
					'checkout_payment',
					$response->ResultCode . ":" . $response->ResultMessage . '<!-- [' . $this->code . '] -->',
					'error'
				);
				zen_redirect(zen_href_link(FILENAME_CHECKOUT_PAYMENT, '', 'SSL', true, false));
			}

			unset($_SESSION[$this->code]['provider']);
			unset($_SESSION[$this->code]['KhbCardPocketId']);
			unset($_SESSION[$this->code]['MkbSzepCafeteriaId']);
			unset($_SESSION[$this->code]['OtpCardNumber']);
			unset($_SESSION[$this->code]['OtpExpiration']);
			unset($_SESSION[$this->code]['OtpCvc']);
			unset($_SESSION[$this->code]['OtpCardPocketId']);
			unset($_SESSION[$this->code]['EscalionOneClickPayment']);
			unset($_SESSION[$this->code]['OTPSimpleOneClickPayment']);
			unset($_SESSION[$this->code]['SaferpayOneClickPayment']);
			unset($_SESSION[$this->code]['SaferpayPaymentMethods']);
			unset($_SESSION[$this->code]['SaferpayWallets']);
			unset($_SESSION[$this->code]['QPAYPaymentTypes']);

			// change order status to value selected by merchant
			$db->Execute(
				"update " . TABLE_ORDERS . " set orders_status = " . (int)MODULE_PAYMENT_BFPG_PENDING_STATUS_ID .
				" where orders_id = " . intval($insert_id)
			);
			$sql = "INSERT INTO " . BIGFISH_CODENAME . "
			(
				orders_id,
				order_total,
				currency,
				provider,
				KhbCardPocketId,
				MkbSzepCafeteriaId,
				OtpCardPocketId,
				MppPhoneNumber,
				TransactionId
			) VALUES (
				:orders_id,
				:order_total,
				:currency,
				:provider,
				:KhbCardPocketId,
				:MkbSzepCafeteriaId,
				:OtpCardPocketId,
				:MppPhoneNumber,
				:TransactionId
			)
			";
			$sql = $db->bindVars($sql, ':orders_id', $data['orderId'], 'integer');
			$sql = $db->bindVars($sql, ':order_total', $data['amount'], 'float');
			$sql = $db->bindVars($sql, ':currency', $data['currency'], 'string');
			$sql = $db->bindVars($sql, ':provider', $data['provider'], 'string');
			$sql = $db->bindVars($sql, ':KhbCardPocketId', $data['extra']['KhbCardPocketId'], 'string');
			$sql = $db->bindVars($sql, ':MkbSzepCafeteriaId', $data['MkbSzepCafeteriaId'], 'string');
			$sql = $db->bindVars($sql, ':OtpCardPocketId', $data['OtpCardPocketId'], 'string');
			$sql = $db->bindVars($sql, ':MppPhoneNumber', '', 'string');
			$sql = $db->bindVars($sql, ':TransactionId', $response->TransactionId, 'string');
			$db->Execute($sql);

			$sql = "UPDATE " . TABLE_ORDERS . " SET orders_status = " . (int)MODULE_PAYMENT_BFPG_PENDING_STATUS_ID .
				" WHERE orders_id = '" . (int)$data['orderId'] . "'";
			$db->Execute($sql);

			$sql_data_array = array(
				'orders_id' => (int)$insert_id,
				'orders_status_id' => (int)MODULE_PAYMENT_BFPG_PENDING_STATUS_ID,
				'date_added' => 'now()',
				'comments' => 'TransactionId: ' . $response->TransactionId,
				'customer_notified' => 0
			);
			zen_db_perform(TABLE_ORDERS_STATUS_HISTORY, $sql_data_array);

			$db->Execute(
				"DELETE FROM " . BIGFISH_CODENAME . "_session
				WHERE session_id = '" . zen_db_input(zen_session_id()) . "'
				OR expiry < " . time()
			);

			$sql = "INSERT INTO " . BIGFISH_CODENAME . "_session
				(session_id, saved_session, expiry) values (
				'" . zen_db_input(zen_session_id()) . "',
				'" . base64_encode(serialize($_SESSION)) . "',
				'" . (time() + (1 * 60 * 60 * 24 * 2)) . "')";
			$db->Execute($sql);

			$_SESSION['cart']->reset(true);

			PaymentGateway::start(new StartRequest($response->TransactionId));

			return true;
		} catch(PaymentGateway_Exception $e) {
			global $messageStack;
			$messageStack->add_session(
				'checkout_payment',
				$e->getMessage() . '<!-- [' . $this->code . '] -->',
				'error'
			);
			zen_redirect(zen_href_link(FILENAME_CHECKOUT_PAYMENT, '', 'SSL', true, false));
		}

		return false;
	}

	/**
	 * get error
	 *
	 * @return bool
	 */
	function get_error()
	{
		return false;
	}

	/**
	 * check
	 *
	 * @return int
	 */
	function check()
	{
		global $db;

		if (IS_ADMIN_FLAG === true) {
			global $sniffer;
			if (!$sniffer->table_exists($this->code)) {
				$db->Execute(
					"
				CREATE TABLE IF NOT EXISTS `" . BIGFISH_CODENAME . "` (
					`id` int(10) unsigned NOT NULL AUTO_INCREMENT,
					`orders_id` int(10) unsigned DEFAULT NULL,
					`order_total` decimal(15,5) unsigned DEFAULT NULL,
					`currency` varchar(4) DEFAULT NULL,
					`provider` varchar(64) DEFAULT NULL,
					`KhbCardPocketId` varchar(64) DEFAULT NULL,
					`MkbSzepCafeteriaId` varchar(4) DEFAULT NULL,
					`OtpCardPocketId` varchar(64) DEFAULT NULL,
					`MppPhoneNumber` varchar(64) DEFAULT NULL,
					`TransactionId` varchar(128) DEFAULT NULL,
					`Anum` varchar(64) DEFAULT NULL,
					`requestAutoCommit` smallint(1) DEFAULT NULL,
					`ResultCode` varchar(128) DEFAULT NULL,
					`ResultMessage` varchar(255) DEFAULT NULL,
					`ProviderTransactionId` varchar(255) DEFAULT NULL,
					`AutoCommit` smallint(1) DEFAULT NULL,
					`CommitState` varchar(64) DEFAULT NULL,
					`OtherData` text,
					PRIMARY KEY (`id`),
					KEY `orders_id` (`orders_id`),
					KEY `TransactionId` (`TransactionId`)
				) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1"
				);
				$db->Execute(
					"
				CREATE TABLE IF NOT EXISTS `" . BIGFISH_CODENAME . "_session` (
					`id` int(10) unsigned NOT NULL AUTO_INCREMENT,
					`session_id` text NULL,
					`saved_session` mediumblob NOT NULL,
					`expiry` int(17) NOT NULL DEFAULT '0',
					PRIMARY KEY (`id`),
					KEY `session_id` (`session_id`(36)),
					KEY `expiry` (`expiry`)
				) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1"
				);
			}
		}

		if (!isset($this->_check)) {
			$check_query = $db->Execute(
				"select configuration_value from " . TABLE_CONFIGURATION .
				" where configuration_key = 'MODULE_PAYMENT_BFPG_STATUS'"
			);
			$this->_check = $check_query->RecordCount();
		}

		return $this->_check;
	}

	/**
	 * install
	 *
	 * @return string
	 */
	function install()
	{
		global $db, $messageStack;

		include_once(DIR_FS_CATALOG_LANGUAGES . $_SESSION['language'] . '/modules/payment/' . $this->code . '.php');

		if (defined('MODULE_PAYMENT_BFPG_STATUS')) {
			$messageStack->add_session(MODULE_PAYMENT_BFPG_TEXT_ALREADY_INSTALLED, 'error');
			zen_redirect(zen_href_link(FILENAME_MODULES, 'set=payment&module=bigfish_paymentgateway', 'SSL'));

			return 'failed';
		}

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_ENABLE_MODUL . "', 'MODULE_PAYMENT_BFPG_STATUS', '" .
			MODULE_PAYMENT_BFPG_TEXT_TRUE .
			"', 'Do you want to accept Big Fish PaymentGateway payments via paymentgateway.hu?', '6', '0', 'zen_cfg_select_option(array(\'" .
			MODULE_PAYMENT_BFPG_TEXT_TRUE . "\', \'" . MODULE_PAYMENT_BFPG_TEXT_FALSE . "\'), ', now());"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_STORENAME .
			"', 'MODULE_PAYMENT_BFPG_STORENAME', '', 'Enter you store name', '6', '0', now());"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_APIKEY .
			"', 'MODULE_PAYMENT_BFPG_APIKEY', '', 'Enter you API Key', '6', '0', now());"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_TRANSACTION_MODE . "', 'MODULE_PAYMENT_BFPG_TESTMODE', '" .
			MODULE_PAYMENT_BFPG_TEXT_TEST .
			"', 'Transaction mode used for processing orders', '6', '0', 'zen_cfg_select_option(array(\'" .
			MODULE_PAYMENT_BFPG_TEXT_TEST . "\', \'" . MODULE_PAYMENT_BFPG_TEXT_PRODUCTION . "\'), ', now())"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_UNPAID_STATUS . "', 'MODULE_PAYMENT_BFPG_PENDING_STATUS_ID', '" .
			(int)DEFAULT_ORDERS_STATUS_ID .
			"', 'Automatically set the status of unpaid orders to this value.', '6', '0', 'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_WAITING_STATUS .
			"', 'MODULE_PAYMENT_BFPG_WAITING_STATUS_ID', '', 'Automatically set the status of waiting orders to this value.', '6', '0', 'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_PAID_STATUS .
			"', 'MODULE_PAYMENT_BFPG_SUCCESS_STATUS_ID', '', 'Automatically set the status of paid orders to this value.', '6', '0', 'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_CANCELLED_STATUS .
			"', 'MODULE_PAYMENT_BFPG_CANCELLED_STATUS_ID', '', 'Automatically set the status of paid orders to this value.', '6', '0', 'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_CURRENCIES .
			"', 'MODULE_PAYMENT_BFPG_CURRENCIES', 'HUF, USD, EUR', 'Only enable Big Fish PaymentGateway payments if one of these currencies is selected (note: currency must be supported by paymentgateway.hu).', '6', '0', now())"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, use_function, set_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_PAYMENT_ZONE .
			"', 'MODULE_PAYMENT_BFPG_ZONE', '0', 'If a zone is selected, only enable this payment method for that zone.', '6', '2', 'zen_get_zone_class_title', 'zen_cfg_pull_down_zone_classes(', now())"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_SETORDER .
			"', 'MODULE_PAYMENT_BFPG_SORT_ORDER', '0', 'Sort order of display. Lowest is displayed first.', '6', '2', now())"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_ENABLE_PROVIDERS .
			"', 'MODULE_PAYMENT_BFPG_PROVIDERS', '', '', '6', '13', 'zen_cfg_select_multioption(array(" .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_CIB . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_KHB_CREDITCARD . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_KHB_SZEP_SZALLAS . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_KHB_SZEP_VENDEGLATAS . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_KHB_SZEP_REKREACIO . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_OTP_CREDITCARD . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_OTP_SZEP_SZALLAS . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_OTP_SZEP_VENDEGLATAS . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_OTP_SZEP_REKREACIO . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_OTPAY . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_OTP2 . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_ESCALION . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_QPAY . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_UNICREDIT . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_FHB . "\',"  .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_PAYPAL . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_PAYU2 . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_SMS . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_SOFORT . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_BORGUN . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_OTPMASTERPASS . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_OTPMULTIPONT . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_MKB_SZEP_SZALLAS . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_MKB_SZEP_VENDEGLATAS . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_MKB_SZEP_REKREACIO . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_PAYSAFECARD . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_OTPSIMPLE . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_OTPSIMPLEWIRE . "\'," .
			"\'" . MODULE_PAYMENT_BFPG_PROVIDER_SAFERPAY . "\'" .
			"),', now());"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) ".
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_OTP_SZEP_STORENAME .
			"', 'MODULE_PAYMENT_BFPG_OTP_SZEP_STORENAME', '', 'Enter your OTP SZÉP Card store name', '6', '0', now());"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) ".
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_OTP_SZEP_APIKEY .
			"', 'MODULE_PAYMENT_BFPG_OTP_SZEP_APIKEY', '', 'Enter your OTP SZÉP CARD API Key', '6', '0', now());"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_AUTOCOMMIT_OTP . "', 'MODULE_PAYMENT_BFPG_AUTOCOMMIT_OTP', '" .
			MODULE_PAYMENT_BFPG_TEXT_ENABLE . "', '', '6', '0', 'zen_cfg_select_option(array(\'" .
			MODULE_PAYMENT_BFPG_TEXT_ENABLE . "\', \'" . MODULE_PAYMENT_BFPG_TEXT_DISABLE . "\'), ', now());"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_AUTOCOMMIT_OTP2 . "', 'MODULE_PAYMENT_BFPG_AUTOCOMMIT_OTP2', '" .
			MODULE_PAYMENT_BFPG_TEXT_ENABLE . "', '', '6', '0', 'zen_cfg_select_option(array(\'" .
			MODULE_PAYMENT_BFPG_TEXT_ENABLE . "\', \'" . MODULE_PAYMENT_BFPG_TEXT_DISABLE . "\'), ', now());"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_AUTOCOMMIT_OTPAY . "', 'MODULE_PAYMENT_BFPG_AUTOCOMMIT_OTPAY', '" .
			MODULE_PAYMENT_BFPG_TEXT_ENABLE . "', '', '6', '0', 'zen_cfg_select_option(array(\'" .
			MODULE_PAYMENT_BFPG_TEXT_ENABLE . "\', \'" . MODULE_PAYMENT_BFPG_TEXT_DISABLE . "\'), ', now());"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_AUTOCOMMIT_QPAY . "', 'MODULE_PAYMENT_BFPG_AUTOCOMMIT_QPAY', '" .
			MODULE_PAYMENT_BFPG_TEXT_ENABLE . "', '', '6', '0', 'zen_cfg_select_option(array(\'" .
			MODULE_PAYMENT_BFPG_TEXT_ENABLE . "\', \'" . MODULE_PAYMENT_BFPG_TEXT_DISABLE . "\'), ', now());"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_AUTOCOMMIT_UNICREDIT .
			"', 'MODULE_PAYMENT_BFPG_AUTOCOMMIT_UNICREDIT', '" . MODULE_PAYMENT_BFPG_TEXT_ENABLE .
			"', '', '6', '0', 'zen_cfg_select_option(array(\'" . MODULE_PAYMENT_BFPG_TEXT_ENABLE . "\', \'" .
			MODULE_PAYMENT_BFPG_TEXT_DISABLE . "\'), ', now());"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_AUTOCOMMIT_FHB . "', 'MODULE_PAYMENT_BFPG_AUTOCOMMIT_FHB', '" .
			MODULE_PAYMENT_BFPG_TEXT_ENABLE . "', '', '6', '0', 'zen_cfg_select_option(array(\'" .
			MODULE_PAYMENT_BFPG_TEXT_ENABLE . "\', \'" . MODULE_PAYMENT_BFPG_TEXT_DISABLE . "\'), ', now());"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_AUTOCOMMIT_PAYU2 . "', 'MODULE_PAYMENT_BFPG_AUTOCOMMIT_PAYU2', '" .
			MODULE_PAYMENT_BFPG_TEXT_ENABLE . "', '', '6', '0', 'zen_cfg_select_option(array(\'" .
			MODULE_PAYMENT_BFPG_TEXT_ENABLE . "\', \'" . MODULE_PAYMENT_BFPG_TEXT_DISABLE . "\'), ', now());"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_PUBLICKEY_OTP2 .
			"', 'MODULE_PAYMENT_BFPG_PUBLICKEY_OTP2', '', '', '6', '0', 'zen_cfg_textarea(', now())"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) ".
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_ONECLICKPAYMENT_ESCALION .
			"', 'MODULE_PAYMENT_BFPG_ONECLICKPAYMENT_ESCALION', '". MODULE_PAYMENT_BFPG_TEXT_DISABLE .
			"', '', '6', '0', 'zen_cfg_select_option(array(\'" . MODULE_PAYMENT_BFPG_TEXT_ENABLE . "\', \'" . MODULE_PAYMENT_BFPG_TEXT_DISABLE .
			"\'), ', now());"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) ".
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_ONECLICKPAYMENT_OTPSIMPLE .
			"', 'MODULE_PAYMENT_BFPG_ONECLICKPAYMENT_OTPSIMPLE', '". MODULE_PAYMENT_BFPG_TEXT_DISABLE .
			"', '', '6', '0', 'zen_cfg_select_option(array(\'" . MODULE_PAYMENT_BFPG_TEXT_ENABLE . "\', \'" . MODULE_PAYMENT_BFPG_TEXT_DISABLE .
			"\'), ', now());"
		);

		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) ".
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_ONECLICKPAYMENT_SAFERPAY .
			"', 'MODULE_PAYMENT_BFPG_ONECLICKPAYMENT_SAFERPAY', '". MODULE_PAYMENT_BFPG_TEXT_DISABLE .
			"', '', '6', '0', 'zen_cfg_select_option(array(\'" . MODULE_PAYMENT_BFPG_TEXT_ENABLE . "\', \'" . MODULE_PAYMENT_BFPG_TEXT_DISABLE .
			"\'), ', now());"
		);

		$providerData = new ProviderData();

		$saferpayPaymentMethods = array();
		$saferpayPaymentMethodsArray = $providerData->getSaferpayPaymentMethodsArray();
		$saferpayWallets = array();
		$saferpayWalletsArray = $providerData->getSaferpayWalletsArray();
		foreach ($saferpayPaymentMethodsArray as $key => $value) {
			$saferpayPaymentMethods[] = "\'" . $value . "\'";
		}
		foreach ($saferpayWalletsArray as $key => $value) {
			$saferpayWallets[] = "\'" . $value . "\'";
		}
		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_SAFERPAY_PAYMENT_METHODS .
			"', 'MODULE_PAYMENT_BFPG_SAFERPAY_PAYMENT_METHODS', '', '', '6', '13', 'zen_cfg_select_multioption(array(" .
			implode(',', $saferpayPaymentMethods) .
			"),', now());"
		);
		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_SAFERPAY_WALLETS .
			"', 'MODULE_PAYMENT_BFPG_SAFERPAY_WALLETS', '', '', '6', '13', 'zen_cfg_select_multioption(array(" .
			implode(',', $saferpayWallets) .
			"),', now());"
		);

		$qpayPaymentTypes = array();
		$qpayPaymentTypesArray = $providerData->getWirecardPaymentTypesArray();
		$qpayPaymentTypes[] = MODULE_PAYMENT_BFPG_PROVIDER_QPAY_SETTING_BY_STORE;
		foreach ($qpayPaymentTypesArray as $key => $value) {
			$qpayPaymentTypes[] = "\'" . $value . "\'";
		}
		$db->Execute(
			"insert into " . TABLE_CONFIGURATION .
			" (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) " .
			"values ('" . MODULE_PAYMENT_BFPG_TEXT_QPAY_PAYMENT_TYPES .
			"', 'MODULE_PAYMENT_BFPG_QPAY_PAYMENT_TYPES', '', '', '6', '13', 'zen_cfg_select_option(array(" .
			implode(',', $qpayPaymentTypes) .
			"),', now());"
		);
	}

	/**
	 * remove
	 */
	function remove()
	{
		global $db;
		$db->Execute(
			"delete from " . TABLE_CONFIGURATION . " where configuration_key in ('" . implode("', '", $this->keys()) .
			"')"
		);
	}

	/**
	 * keys
	 *
	 * @return array
	 */
	function keys()
	{
		return array(
			'MODULE_PAYMENT_BFPG_STATUS',
			'MODULE_PAYMENT_BFPG_STORENAME',
			'MODULE_PAYMENT_BFPG_APIKEY',
			'MODULE_PAYMENT_BFPG_TESTMODE',
			'MODULE_PAYMENT_BFPG_PENDING_STATUS_ID',
			'MODULE_PAYMENT_BFPG_WAITING_STATUS_ID',
			'MODULE_PAYMENT_BFPG_SUCCESS_STATUS_ID',
			'MODULE_PAYMENT_BFPG_CANCELLED_STATUS_ID',
			'MODULE_PAYMENT_BFPG_SORT_ORDER',
			'MODULE_PAYMENT_BFPG_ZONE',
			'MODULE_PAYMENT_BFPG_CURRENCIES',
			'MODULE_PAYMENT_BFPG_PROVIDERS',
			'MODULE_PAYMENT_BFPG_AUTOCOMMIT_OTP',
			'MODULE_PAYMENT_BFPG_AUTOCOMMIT_OTPAY',
			'MODULE_PAYMENT_BFPG_AUTOCOMMIT_OTP2',
			'MODULE_PAYMENT_BFPG_AUTOCOMMIT_QPAY',
			'MODULE_PAYMENT_BFPG_AUTOCOMMIT_UNICREDIT',
			'MODULE_PAYMENT_BFPG_AUTOCOMMIT_FHB',
			'MODULE_PAYMENT_BFPG_AUTOCOMMIT_PAYU2',
			'MODULE_PAYMENT_BFPG_PUBLICKEY_OTP2',
			'MODULE_PAYMENT_BFPG_ONECLICKPAYMENT_ESCALION',
			'MODULE_PAYMENT_BFPG_ONECLICKPAYMENT_OTPSIMPLE',
			'MODULE_PAYMENT_BFPG_ONECLICKPAYMENT_SAFERPAY',
			'MODULE_PAYMENT_BFPG_OTP_SZEP_STORENAME',
			'MODULE_PAYMENT_BFPG_OTP_SZEP_APIKEY',
			'MODULE_PAYMENT_BFPG_SAFERPAY_PAYMENT_METHODS',
			'MODULE_PAYMENT_BFPG_SAFERPAY_WALLETS',
			'MODULE_PAYMENT_BFPG_QPAY_PAYMENT_TYPES'
		);
	}

	/**
	 * admin notification in order datas
	 *
	 * @param $zf_order_id
	 *
	 * @return string
	 */
	function admin_notification($zf_order_id)
	{
		global $db;
		$output = '<td><table>';

		$sql = "SELECT * FROM " . BIGFISH_CODENAME . " WHERE orders_id=:orders_id ";
		$sql = $db->bindVars($sql, ':orders_id', $zf_order_id, 'string');
		/** @var object $result */
		$result = $db->Execute($sql, 1);

		$this->loadProviderData();

		$output .= '<tr><td>' . MODULE_PAYMENT_BFPG_TEXT_PROVIDER . ': ' . $this->providerData['provider'][$result->fields['provider']]['title'] . '</td></tr>';

		if (!empty($result->fields['OtpCardPocketId'])) {
			$pocketName = $this->providerData['OtpCardPocketId'][$result->fields['provider'] .
			$result->fields['OtpCardPocketId']]['title'];
			$output .= '<tr><td>' . MODULE_PAYMENT_BFPG_TEXT_POCKET . ': ' . $result->fields['OtpCardPocketId'] . '(' .
				$pocketName . ')</td></tr>';
		}
		if (!empty($result->fields['KhbCardPocketId'])) {
			$pocketName = $this->providerData['KhbCardPocketId'][$result->fields['provider'] .
			$result->fields['KhbCardPocketId']]['title'];
			$output .= '<tr><td>' . MODULE_PAYMENT_BFPG_TEXT_POCKET . ': ' . $result->fields['KhbCardPocketId'] . '(' .
				$pocketName . ')</td></tr>';
		}
		if (!empty($result->fields['MkbSzepCafeteriaId'])) {
			$pocketName = $this->providerData['MkbSzepCafeteriaId'][$result->fields['provider'] .
			$result->fields['MkbSzepCafeteriaId']]['title'];
			$output .= '<tr><td>' . MODULE_PAYMENT_BFPG_TEXT_POCKET . ': ' . $result->fields['MkbSzepCafeteriaId'] .
				'(' . $pocketName . ')</td></tr>';
		}
		if (!empty($result->fields['Anum'])) {
			$output .= '<tr><td>' . MODULE_PAYMENT_BFPG_TEXT_ANUM . ': ' . $result->fields['Anum'] . '</td></tr>';
		}
		if (!empty($result->fields['ProviderTransactionId'])) {
			$output .= '<tr><td>' . MODULE_PAYMENT_BFPG_TEXT_PTID . ': ' . $result->fields['ProviderTransactionId'] .
				'</td></tr>';
		}

		$output .= '</table></td>';

		return $output;
	}

	/**
	 * _doStatusUpdate method
	 * called after admin status update
	 *
	 * @param $zf_order_id
	 * @param $newStatus
	 * @param $comments
	 * @param $customerNotified
	 * @param $oldStatus
	 *
	 * @return bool
	 */
	function _doStatusUpdate($zf_order_id, $newStatus, $comments, $customerNotified, $oldStatus)
	{
		global $db, $messageStack;

		if ($oldStatus == MODULE_PAYMENT_BFPG_WAITING_STATUS_ID && in_array(
				$newStatus,
				array(
					MODULE_PAYMENT_BFPG_CANCELLED_STATUS_ID,
					MODULE_PAYMENT_BFPG_SUCCESS_STATUS_ID,
				)
			)
		) {
			$sql = "SELECT * FROM " . BIGFISH_CODENAME . " WHERE orders_id=:orders_id ";
			$sql = $db->bindVars($sql, ':orders_id', $zf_order_id, 'string');
			/** @var object $result */
			$result = $db->Execute($sql, 1);
			if ($result->RecordCount() == 0) {
				$messageStack->add_session(
					sprintf(MODULE_PAYMENT_BFPG_TEXT_ERROR_NO_DATA, $zf_order_id),
					'error'
				);
			}
			if ($result->fields['AutoCommit'] != '0' || $result->fields['CommitState'] == 'APPROVED') {
				$messageStack->add_session(
					sprintf(MODULE_PAYMENT_BFPG_TEXT_ERROR_APPROVED, $zf_order_id),
					'error'
				);
			}

			$approved = null;
			if ($newStatus == MODULE_PAYMENT_BFPG_CANCELLED_STATUS_ID) {
				$approved = false;
			} else {
				if ($newStatus == MODULE_PAYMENT_BFPG_SUCCESS_STATUS_ID) {
					$approved = true;
				}
			}
			if ($approved !== null) {
				$this->setPaymentGatewayConfig($result->fields['provider']);
				try {
					$response = PaymentGateway::close(new CloseRequest($result->fields['TransactionId'], $approved));
					if ($response->ResultCode == PaymentGateway::RESULT_CODE_SUCCESS) {
						$response = PaymentGateway::details(new DetailsRequest($result->fields['TransactionId']));

						$sql = "
						UPDATE
						" . BIGFISH_CODENAME . "
						SET
						Anum=:Anum,
						ResultCode=:ResultCode,
						ResultMessage=:ResultMessage,
						ProviderTransactionId=:ProviderTransactionId,
						AutoCommit=:AutoCommit,
						CommitState=:CommitState,
						OtherData=:OtherData
						WHERE
						orders_id = :orders_id ";
						$sql = $db->bindVars($sql, ':Anum', $response->CommonData->Anum, 'string');
						$sql = $db->bindVars($sql, ':ResultCode', $response->CommonData->ResultCode, 'string');
						$sql = $db->bindVars($sql, ':ResultMessage', $response->CommonData->ResultMessage, 'string');
						$sql = $db->bindVars(
							$sql,
							':ProviderTransactionId',
							$response->CommonData->ProviderTransactionId,
							'string'
						);
						$sql = $db->bindVars($sql, ':AutoCommit', $response->CommonData->AutoCommit, 'integer');
						$sql = $db->bindVars($sql, ':CommitState', $response->CommonData->CommitState, 'string');
						$sql = $db->bindVars($sql, ':OtherData', $response->CommonData->OtherData, 'string');
						$sql = $db->bindVars($sql, ':orders_id', $zf_order_id, 'integer');
						$db->Execute($sql);

						$sql_data_array = array(
							'orders_id' => (int)$zf_order_id,
							'orders_status_id' => (int)$newStatus,
							'date_added' => 'now()',
							'comments' => var_export(get_object_vars($response->CommonData), true),
							'customer_notified' => 0
						);
						zen_db_perform(TABLE_ORDERS_STATUS_HISTORY, $sql_data_array);

						return true;
					}
				} catch(PaymentGateway_Exception $e) {
					$messageStack->add_session(
						sprintf(MODULE_PAYMENT_BFPG_TEXT_ERROR_OCCURED, $zf_order_id, $e->getMessage()),
						'error'
					);

					return false;
				}
			}
		} else {
			if ($newStatus == MODULE_PAYMENT_BFPG_WAITING_STATUS_ID && in_array(
					$oldStatus,
					array(
						MODULE_PAYMENT_BFPG_CANCELLED_STATUS_ID,
						MODULE_PAYMENT_BFPG_SUCCESS_STATUS_ID,
					)
				)
			) {
				$messageStack->add_session(
					sprintf(MODULE_PAYMENT_BFPG_TEXT_ERROR_APPROVED, $zf_order_id),
					'error'
				);

				return false;
			}
		}

		return true;
	}

	/**
	 * load provider data
	 */
	function loadProviderData()
	{
		$this->providerData['method'] = new ProviderData();

		$this->providerData['provider'] = $this->providerData['method']->getVar('provider');
		$this->providerData['countProvider'] = count($this->providerData['provider']);
		$this->providerData['KhbCardPocketId'] = $this->providerData['method']->getVar('KhbCardPocketId');
		$this->providerData['countKhbCardPocketId'] = count($this->providerData['KhbCardPocketId']);
		$this->providerData['MkbSzepCafeteriaId'] = $this->providerData['method']->getVar('MkbSzepCafeteriaId');
		$this->providerData['countMkbSzepCafeteriaId'] = count($this->providerData['MkbSzepCafeteriaId']);
		$this->providerData['OtpCardPocketId'] = $this->providerData['method']->getVar('OtpCardPocketId');
		$this->providerData['countOtpCardPocketId'] = count($this->providerData['OtpCardPocketId']);
		$this->providerData['requestAutoCommit'] = $this->providerData['method']->getVar('requestAutoCommit');
		$this->providerData['CreditCard'] = $this->providerData['method']->getVar('CreditCard');
		$this->providerData['EscalionOneClickPayment'] = $this->providerData['method']->getVar('escalionOneClickPayment');
		$this->providerData['OTPSimpleOneClickPayment'] = $this->providerData['method']->getVar('otpSimpleOneClickPayment');
		$this->providerData['SaferpayOneClickPayment'] = $this->providerData['method']->getVar('saferpayOneClickPayment');
		$this->providerData['SaferpayPaymentMethods'] = $this->providerData['method']->getVar('saferpayPaymentMethods');
		$this->providerData['SaferpayWallets'] = $this->providerData['method']->getVar('saferpayWallets');
		$this->providerData['QPAYPaymentTypes'] = $this->providerData['method']->getVar('qpayPaymentTypes');
	}

	/**
	 * set payment gateway config
	 * @param string $provider
	 * @param bool $isOTPSZEPCard
	 */
	function setPaymentGatewayConfig($provider = '', $isOTPSZEPCard = false) {
		$configData = array(
			'apiKey' => ($isOTPSZEPCard && MODULE_PAYMENT_BFPG_OTP_SZEP_APIKEY != '' ? MODULE_PAYMENT_BFPG_OTP_SZEP_APIKEY : MODULE_PAYMENT_BFPG_APIKEY),
			'testMode' => (MODULE_PAYMENT_BFPG_TESTMODE == MODULE_PAYMENT_BFPG_TEXT_TEST ? true : false),
			'storeName' => ($isOTPSZEPCard && MODULE_PAYMENT_BFPG_OTP_SZEP_STORENAME != '' ? MODULE_PAYMENT_BFPG_OTP_SZEP_STORENAME : MODULE_PAYMENT_BFPG_STORENAME),
			'moduleName' => BIGFISH_MODULE_NAME . ' (' . PROJECT_VERSION_MAJOR . '.' . PROJECT_VERSION_MINOR .
				(PROJECT_VERSION_PATCH1 != '' ? 'p' . PROJECT_VERSION_PATCH1 : '') . ')',
			'moduleVersion' => BIGFISH_MODULE_VERSION
		);
		if ($provider == 'OTP2') {
			$configData['encryptPublicKey'] = MODULE_PAYMENT_BFPG_PUBLICKEY_OTP2;
		}
		$config = new Config($configData);
		PaymentGateway::setConfig($config);
	}
}